@@ -49,7 +49,7 @@ void extract2(const Size2D &size,
 {
     internal::assertSupportedConfiguration();
 #ifdef CAROTENE_NEON
-#ifndef ANDROID
+#ifndef __ANDROID__
     size_t roiw32 = size.width >= 31 ? size.width - 31 : 0;
 #endif
     size_t roiw8 = size.width >= 7 ? size.width - 7 : 0;
@@ -60,7 +60,7 @@ void extract2(const Size2D &size,
         u8 * dst = internal::getRowPtr(dstBase, dstStride, i);
         size_t sj = 0u, dj = 0u;
 
-#ifndef ANDROID
+#ifndef __ANDROID__
         for (; dj < roiw32; sj += 64, dj += 32)
         {
             internal::prefetch(src + sj);
@@ -101,7 +101,7 @@ void extract3(const Size2D &size,
 {
     internal::assertSupportedConfiguration();
 #ifdef CAROTENE_NEON
-#ifndef ANDROID
+#ifndef __ANDROID__
     size_t roiw32 = size.width >= 31 ? size.width - 31 : 0;
 #endif
     size_t roiw8 = size.width >= 7 ? size.width - 7 : 0;
@@ -112,7 +112,7 @@ void extract3(const Size2D &size,
         u8 * dst = internal::getRowPtr(dstBase, dstStride, i);
         size_t sj = 0u, dj = 0u;
 
-#ifndef ANDROID
+#ifndef __ANDROID__
         for (; dj < roiw32; sj += 96, dj += 32)
         {
             internal::prefetch(src + sj);
@@ -153,7 +153,7 @@ void extract4(const Size2D &size,
 {
     internal::assertSupportedConfiguration();
 #ifdef CAROTENE_NEON
-#ifndef ANDROID
+#ifndef __ANDROID__
     size_t roiw32 = size.width >= 31 ? size.width - 31 : 0;
 #endif
     size_t roiw8 = size.width >= 7 ? size.width - 7 : 0;
@@ -164,7 +164,7 @@ void extract4(const Size2D &size,
         u8 * dst = internal::getRowPtr(dstBase, dstStride, i);
         size_t sj = 0u, dj = 0u;
 
-#ifndef ANDROID
+#ifndef __ANDROID__
         for (; dj < roiw32; sj += 128, dj += 32)
         {
             internal::prefetch(src + sj);
@@ -231,7 +231,7 @@ void extract4(const Size2D &size,
                  srcStride == dst2Stride && \
                  srcStride == dst3Stride &&
 
-#if __GNUC__ == 4 && __GNUC_MINOR__ < 7
+#if !defined(__aarch64__) && defined(__GNUC__) && __GNUC__ == 4 &&  __GNUC_MINOR__ < 7 && !defined(__clang__)
 
 #define SPLIT_ASM2(sgn, bits) __asm__ ( \
                                           "vld2." #bits " {d0, d2}, [%[in0]]            \n\t" \
@@ -280,7 +280,7 @@ void extract4(const Size2D &size,
                                      FILL_LINES##n(VST1Q, sgn##bits) \
                                  }
 
-#endif // __GNUC__ == 4 && __GNUC_MINOR__ < 7
+#endif
 
 #define SPLIT(sgn,bits,n) void split##n(const Size2D &_size,                                            \
                                     const sgn##bits * srcBase, ptrdiff_t srcStride                      \
@@ -351,7 +351,7 @@ void extract4(const Size2D &size,
     }                                                                                                   \
 }
 
-#if __GNUC__ == 4 && __GNUC_MINOR__ < 7
+#if !defined(__aarch64__) && defined(__GNUC__) && __GNUC__ == 4 &&  __GNUC_MINOR__ < 7 && !defined(__clang__)
 
 #define ALPHA_QUAD(sgn, bits) { \
                                   internal::prefetch(src + sj); \
@@ -378,7 +378,7 @@ void extract4(const Size2D &size,
                                   vst1q_##sgn##bits(dst1 + d1j, vals.v4.val[3]); \
                               }
 
-#endif // __GNUC__ == 4 && __GNUC_MINOR__ < 7
+#endif 
 
 #define SPLIT4ALPHA(sgn,bits) void split4(const Size2D &_size,                                          \
                                           const sgn##bits * srcBase, ptrdiff_t srcStride,               \
